@@ -3,6 +3,7 @@
  *
  * SPDX-License-Identifier: LGPL-3.0-or-later
  */
+#include "configure.h"
 #include "linglong/api/dbus/v1/dbus_peer.h"
 #include "linglong/cli/cli.h"
 #include "linglong/cli/cli_printer.h"
@@ -13,7 +14,6 @@
 #include "linglong/repo/config.h"
 #include "linglong/repo/ostree_repo.h"
 #include "linglong/runtime/container_builder.h"
-#include "linglong/utils/configure.h"
 #include "linglong/utils/finally/finally.h"
 #include "linglong/utils/gettext.h"
 #include "linglong/utils/global/initialize.h"
@@ -119,10 +119,11 @@ int lockCheck() noexcept
         ::close(fd);
     });
 
-    struct flock lock_info
-    {
-        .l_type = F_RDLCK, .l_whence = SEEK_SET, .l_start = 0, .l_len = 0, .l_pid = 0
-    };
+    struct flock lock_info{ .l_type = F_RDLCK,
+                            .l_whence = SEEK_SET,
+                            .l_start = 0,
+                            .l_len = 0,
+                            .l_pid = 0 };
 
     if (::fcntl(fd, F_GETLK, &lock_info) == -1) {
         qCritical() << "failed to get lock" << lock;
@@ -202,7 +203,10 @@ You can report bugs to the linyaps team under this project: https://github.com/O
                         .showAllVersion = false,
                         .showUpgradeList = false,
                         .forceOpt = false,
-                        .confirmOpt = false };
+                        .confirmOpt = false,
+                        .verbose = false };
+
+    commandParser.add_flag("-v,--verbose", options.verbose, _("Show debug info (verbose logs)"));
 
     // groups
     auto *CliBuildInGroup = _("Managing installed applications and runtimes");
@@ -236,6 +240,18 @@ ll-cli run org.deepin.demo -- bash -x /path/to/bash/script)"));
       ->add_option("--url", options.fileUrls, _("Pass url to applications running in a sandbox"))
       ->type_name("URL")
       ->expected(0, -1);
+    cliRun->add_option("--env", options.envs, _("Set environment variables for the application"))
+      ->type_name("ENV")
+      ->expected(0, -1)
+      ->check([](const std::string &env) -> std::string {
+          if (env.find('=') == std::string::npos) {
+              return std::string{ _(
+                "Input parameter is invalid, please input valid parameter instead") };
+          }
+
+          return {};
+      });
+
     cliRun->add_option("COMMAND", options.commands, _("Run commands in a running sandbox"));
 
     // add sub command ps
@@ -321,6 +337,9 @@ ll-cli install stable:org.deepin.demo/0.0.0.1/x86_64
     cliInstall->add_option("--module", options.module, _("Install a specify module"))
       ->type_name("MODULE")
       ->check(validatorString);
+    cliInstall->add_option("--repo", options.repo, _("Install from a specific repo"))
+      ->type_name("REPO")
+      ->check(validatorString);
     cliInstall->add_flag("--force", options.forceOpt, _("Force install the application"));
     cliInstall->add_flag("-y", options.confirmOpt, _("Automatically answer yes to all questions"));
 
@@ -357,7 +376,7 @@ ll-cli install stable:org.deepin.demo/0.0.0.1/x86_64
     cliUpgrade
       ->add_option("APP",
                    options.appid,
-                   _("Specify the application ID.If it not be specified, all "
+                   _("Specify the application ID. If it not be specified, all "
                      "applications will be upgraded"))
       ->check(validatorString);
 
@@ -390,6 +409,9 @@ ll-cli search . --type=runtime)"));
       ->type_name("TYPE")
       ->capture_default_str()
       ->check(validatorString);
+    cliSearch->add_option("--repo", options.repo, _("Specify the repo"))
+      ->type_name("REPO")
+      ->check(validatorString);
     cliSearch->add_flag("--dev", options.showDevel, _("Include develop application in result"));
     cliSearch->add_flag("--show-all-version",
                         options.showAllVersion,
